%This file solves the incomplete markets model with the same beta for
%skilled and unskilled.

%This version has non-linear income taxes and thus needs to solve a
%non-linear equation in c and l.

%It uses the endogenous grid method and does everything in matrix form for any number of shocks.

%Inner loop:
%Fixes labor policies.
%Uses matlab to find the equilibrium prices, transfers, B and calibrates
%omega, nu, mu as well as A SINGLE beta (NO DIFFERENCES) to match the approprite targets.
%It sets G as as a fraction of output (gy = 16% in benchmark).

%Outer loop:
%Find labor policies consistent with consumption.

%clc
clear all
warning off all
%profile on

tic;
global nA A a m alpha eta omega rho nu k0 ds de lu ls rs re tauke tauks ks ke taul nY sigma Yu Ys asprime auprime ...
    lambdas lambdau lambdasa lambdaua gy x pis piu Ts Tu Amat Tlambdau Tlambdas tauc by lambdal lspol lupol lsinv luinv  cspol cupol

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sigma = 2; %CRRA in consumption
gamma = 1; %power disutility of labor
taul = 0.18; %progressivity tax parameter in HSV

gy = 0.16; %share of G in Y
%gy = 0.24; %share of G in Y
by = 0.6; %government debt s share of gdp

%technology
eta = .401; %KORV
rho = -.495;%KORV
alpha = .117; %KORV
de = .124;%GHK
ds = .056;%GHK

%taxes with a 50% bonus depreciation
tauke = .2775; 
tauks = .3965;

tauc = 0.05;

amin = 0;
amax = 10;
%A = linspace(amin, amax, 201)'
%controlling the density of grid points close to 0
A = ((exp(linspace(0,log(exp(2)*amax+1), 101)) - 1)/exp(2))';
nA=length(A);

%total measure of agents of each type calibrated from Census
pis = 0.3169;
piu = 1-pis;

%transition matrix for skills from Krueger-Ludwig, 
perss = .969;
persu = .928;
vars = .01;
varu = .0192;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%discretizing the shock processes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nY = 4;

%rouwenhorst
rouw = rouwenhorst(perss, vars^.5, nY);
Ys = exp(rouw(1,:));
Ts = rouw(2:nY+1, :);
rouw = rouwenhorst(persu, varu^.5, nY);
Yu = exp(rouw(1,:));
Tu = rouw(2:nY+1, :);
%rescaling so that means = 1
options.disp = 0;
[vector value] = eigs(Ts',1,1.1, options);
pisSS = vector(:,1)/sum(vector(:,1))';
[vector value] = eigs(Tu',1,1.1, options);
piuSS = vector(:,1)/sum(vector(:,1))';
meanS = Ys*pisSS;
meanU = Yu*piuSS;
Ys = Ys/meanS;
Yu = Yu/meanU;
cdfs = cumsum(pisSS);
cdfu = cumsum(piuSS);
Tscdf = cumsum(Ts,2);
Tucdf = cumsum(Tu,2);
Amat = kron(A, ones(1,nY));


%finding the wage gini
 wprob = [[1.8*Ys';Yu'],[pisSS/2;piuSS/2]];
 wprobsort = sortrows(wprob,1);
 wage = wprobsort(:,1).*wprobsort(:,2);
 si = cumsum(wage,1);
 probcdf = cumsum(wprobsort(:,2));
 plot(probcdf, si);
 siminus = [0; si(1:2*nY-1)];
 Gini = 1 - (wprobsort(:,2)'*(si+siminus))/si(2*nY);
 
%check properties of the processes approximated by rouwenhorst
simulation = 0;
if simulation ==1;
simuln = 10000;


 %cdfs = cdfu;
 %Tscdf = Tucdf;
random(1) = unifrnd(0,1);
if random(1) < cdfs(1)
    Ysreal(1) = log(Ys(1));
    seps(1) = 1;
else
    for i = 2:nY
        if random(1) >= cdfs(i-1) & random(1)<cdfs(i)
           Ysreal(1) = log(Ys(i));
           seps(1) = i;
        end
    end
end

for j = 2: simuln
    
    random(j) = unifrnd(0,1);

    if random(j) <Tscdf(seps(j-1),1)
       Ysreal(j) = log(Ys(1));
       seps(j) = 1;
    else
        for i = 2:nY
            if random(j) >= Tscdf(seps(j-1), i-1) & random(j)<Tscdf(seps(j-1),i)
                Ysreal(j) = log(Ys(i));
                seps(j) = i;
            end
        end 
    end
end
%plot(Ysreal)
Ysreal = Ysreal';
seps;

 temprho = regress(Ysreal(2:simuln), [ones(simuln-1,1), Ysreal(1:simuln-1)])
 tempres = Ysreal(2:simuln)- temprho(1)*ones(simuln-1,1) - temprho(2)*Ysreal(1:simuln-1);
 (std(tempres))^2

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%initial guesses
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%loading prices etc.
loading = 1;
if loading == 1;
    load z z
 
    z0 = z;
    load asprime asprime
    load auprime auprime
    load lspol lspol
    load lupol lupol
    load lsinv lsinv
    load luinv luinv
    
else %if not a good initial guess available
    betas = .982;
    betau = .982;
    omega = .3; 
    nu = .8;

    lambdasmat = pis*ones(nA, nY)/(nA*nY);
    lambdaumat = piu*ones(nA, nY)/(nA*nY);
    
    asprime = .9*kron(A, ones(1,nY)); %initial guess on policies 
    auprime = .9*kron(A, ones(1,nY)); %initial guess on policies 

    %initial guesses for the first loop
    lspol = 1/3*ones(nA, nY);
    lupol = 1/3*ones(nA, nY);
    lsinv = lspol;
    luinv = lupol;
    %efficient labor units
    ls = sum(sum(kron(Ys, ones(nA,1)).*lambdasmat.*lspol));
    lu = sum(sum(kron(Yu, ones(nA,1)).*lambdaumat.*lupol));
    maxr = (1/betas - 1)/(1-tauks) + ds;
    r = .95*maxr;    %initial guess on structure interest rate
    if r>maxr
        display 'The initial r is not in a interval guaranteeing stationary distribution'   
    end
    rs = r;
    re = (1-tauks)*(rs - ds)/(1-tauke) + de;%the NAC
    options = optimset('display', 'off', 'tolfun', 10^(-20), 'tolX', 10^(-20), 'maxfunevals', 1000000, 'maxiter', 1000000); 
    k = fsolve(@capital, [1 1], options);
    %capital
    k0 = k(1) + k(2);
    ks = k(1);
    ke = k(2);
    x = [ks, ke, lu, ls];
    MP = ycompprime2(x);
    %r = MP(1) %consistency check
    w = [MP(3), MP(4)];
    gdp = ycomp(x); 
    B = by*gdp; 
    
   z0 =  [r, w, lambdal, B, betas, omega, nu, betau, meany];   
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%loop in labor income to get consistency between exogeneous and endogeneous labor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
diffl = 1;
xl = 1;
norml = 10^-9;

while (diffl > norml) & (xl < 50) %iteration on labor income
    
tic; %to measure the time
options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 20); 
%options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 20, 'NonlEqnAlgorithm', 'gn'); 
%options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 20, 'NonlEqnAlgorithm', 'lm'); 
%options = optimset('display', 'iter', 'tolfun', 10^(-10), 'tolX', 10^(-10), 'maxfunevals', 1000000, 'maxiter', 20, 'largescale', 'on'); 

%this function solves the 'inner loop' finds prices and calibrates parameters holding labor policies fixed
z = fsolve(@aiyagarical4, z0, options);
z0 = z;
%aiyagarical4(z0); %just a check
%z = z0;
toc;

if auprime(2,1) == 0;
    auprime(2,1) = 10^-10;
end


%defining variables
rs = z(1);
r = rs;
re = (1-tauks)*(rs - ds)/(1-tauke) + de;%the NAC
w = z(2:3);
lambdal = z(4);
B = z(5);
betas = z(6);
omega = z(7);
nu = z(8);
betau = betas;
meany = z(9);

%stationary distribution
lambdasmat = (reshape(lambdas, nY, nA))';
lambdaumat = (reshape(lambdau, nY, nA))';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%constructing labor policies that would be consistent with consumption, the outer loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computing consumption policies
%these are input into the intra FOC 

Ysprime = lambdal*(kron(Ys*w(2), ones(nA,1)).*lspol/meany).^(-taul).*kron(Ys*w(2), ones(nA,1))*(1-taul);
Yuprime = lambdal*(kron(Yu*w(1), ones(nA,1)).*lspol/meany).^(-taul).*kron(Yu*w(1), ones(nA,1))*(1-taul);


%if gamma ~= 1
%    display 'check for gamma neq 1'
%end
lspoltemp = (Ysprime./((1+tauc)*cspol.^sigma)).^(1/gamma);
lupoltemp = (Yuprime./((1+tauc)*cupol.^sigma)).^(1/gamma);
laggtemp= sum(sum(lambdasmat.*lspoltemp)) + sum(sum(lambdaumat.*lupoltemp));
%calibrating phi
phi = (3*laggtemp)^gamma;
lspolnew = (Ysprime./(phi*(1+tauc)*cspol.^sigma)).^(1/gamma);
lupolnew = (Yuprime./(phi*(1+tauc)*cupol.^sigma)).^(1/gamma);
laggtemp= sum(sum(lambdasmat.*lspolnew)) + sum(sum(lambdaumat.*lupolnew));


diffl = sum(sum(lambdasmat.*abs(lspolnew - lspol))) + sum(sum(lambdaumat.*abs(lupolnew - lupol)))
if diffl > norml
    %setting the polices for next round
    polcoeff = 0.5;
    lspol = polcoeff*lspolnew+(1-polcoeff)*lspol;
    lupol = polcoeff*lupolnew+(1-polcoeff)*lupol;

   %Constructing l(a') to be consistent with both the new l(a,z) and a'(a,z)
    for k = 1:nA
        for m = 1:nY
           lsinvnew(k,m) = interp1(asprime(:, m), lspol(:,m), A(k),'linear','extrap'); %not ignoring the BC, but can have only 1 guy on it
           luinvnew(k,m) = interp1(auprime(:, m), lupol(:,m), A(k),'linear','extrap'); %not ignoring the BC, but can have only 1 guy on it
           
           %linv(k,m) = interp1(aprimeraw(:, m), lpol(:,m), A(k),'linear','extrap'); %ignoring the BC
           %linv(k,m) = interp1(aprime(:, m), lpol(:,m), A(k), 'linear','extrap'); 
           %using linear interpolation in Matlab, doesn't work with binding BC
           %linv(k,m) = interp1(A, lpol(:,m), ainv(k,m), 'linear', 'extrap'); %alternative mapping between l(a,z) and a'(a,z)
        end
    end

%setting the polices for next round
lsinv = lsinvnew;
luinv = luinvnew;
end

saving = 0;
if saving == 1;
    
    save z z
    save asprime asprime
    save auprime auprime
    save lspol lspol
    save lupol lupol
    save lsinv lsinv
    save luinv luinv
end

xl = xl + 1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computing stuff once everything has converged
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%aggregate effective labors
ls = sum(sum(kron(Ys, ones(nA,1)).*lambdasmat.*lspol));
lu = sum(sum(kron(Yu, ones(nA,1)).*lambdaumat.*lupol));

%average labor
l1 = sum(sum(lambdasmat.*lspol))/pis;
l2 = sum(sum(lambdaumat.*lupol))/piu;
lagg = pis*l1+piu*l2;
%lagg = sum(sum(lambdasmat.*lspol)) + sum(sum(lambdaumat.*lupol));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%checking feasibility and a few other things
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%cdf
lambdasmatcdf = cumsum(lambdasmat,1);
lambdaumatcdf = cumsum(lambdaumat,1);
lambdasacdf = cumsum(lambdasa);
lambdauacdf = cumsum(lambdaua);

c1 = sum(sum(lambdasmat.*cspol))/pis;
c2 = sum(sum(lambdaumat.*cupol))/piu;
cagg = sum(sum(lambdasmat.*cspol)) + sum(sum(lambdaumat.*cupol));
iagg = de*ke + ds*ks;

gdp = ycomp(x);
check1 = gdp - lu*w(1) - ls*w(2) - rs*ks - re*ke;
G = gy*gdp;
check2 = gdp-cagg - de*ke - ds*ks - G;

MP = ycompprime2(x);
w = [MP(3), MP(4)];

lagg;
sp = w(2)/w(1); %if the mean is 1 in both sectors - should weight by hours?
KtoY = (ke+ks)/gdp;
lshare = (lu*w(1) + ls*w(2))/gdp;

NFAP = lambdasa'*A + lambdaua'*A - B - (ks+ke);
%domestic asset holdings
assets = lambdasa'*A + lambdaua'*A;

%pre tax earnings
ears = kron(Ys.*w(2),ones(nA,1)).*lspol;
earu = kron(Yu.*w(1),ones(nA,1)).*lupol;
aears = sum(sum(lambdasmat.*ears))/pis;
aearu = sum(sum(lambdaumat.*earu))/piu;
%aear = aears*pis + aearu*piu; %should be equal to meany

%aftertax earnings
Ysmat = lambdal*(kron(Ys*w(2), ones(nA,1)).*lspol/meany).^(1-taul)*meany;
earsat = Ysmat;
Yumat = lambdal*(kron(Yu*w(1), ones(nA,1)).*lupol/meany).^(1-taul)*meany;
earuat = Yumat;
%average taxes
ats  = (ears-Ysmat)./ears;
atu  = (earu-Yumat)./earu;
ats = sum(sum(lambdasmat.*ats))/pis;
atu = sum(sum(lambdaumat.*atu))/piu;
atagg = ats*pis + atu*piu;

%total pre-tax income 
incomes = ears + (rs-ds)*Amat;
incomeu = earu + (rs-ds)*Amat;

%tax revenues
ltax = sum(sum((ears-Ysmat).*lambdasmat +(earu-Yumat).*lambdaumat));

ctax = (MP(1) - ds)*ks*tauks +  (MP(2) - de)*ke*tauke;
corptax = ((1-(1-tauks)/.85)*(MP(1) - ds)*ks + (1-(1-tauke)/.85)*(MP(2) - de)*ke);
%tax rates faced by the firms for consistency
taufe = 1-(1-tauke)/.85;
taufs = 1-(1-tauks)/.85;
cctax = ((1-taufs)*(MP(1) - ds)*ks + (1-taufe)*(MP(2) - de)*ke)*.15;

%compute relative capital holdings - how do KL (2013) do on this?
rks = lambdasa'*A/pis/(lambdaua'*A/piu);

%finding the asset gini
 lambdaa = lambdaua + lambdasa;
 asset = A.*lambdaa;
 si = cumsum(asset,1);
 siminus = [0; si(1:nA-1)];
 Gini = 1 - (lambdaa'*(si+siminus))/si(nA);
% 
% %finding the asset quintiles
 lambdaacdf = lambdauacdf + lambdasacdf;
 quint = lambdaacdf*ones(1,4) - kron([.2,.4,.6,.8], ones(nA,1));
 quint = quint < 0;
 %(A*ones(1,4)).*quint.*(lambdaa*ones(1,4));
 quintiles = [sum((A*ones(1,4)).*quint.*(lambdaa*ones(1,4)))/si(nA),1];
 quintiles = quintiles - [0, quintiles(1:4)];
% 
% %finding the earnings gini - pre-tax, after-tax, include capital income???
 earprob = [[reshape(incomes', nA*nY,1);reshape(incomeu', nA*nY,1)], [lambdas; lambdau]];
 earprobsort = sortrows(earprob,1);
 earning = earprobsort(:,1).*earprobsort(:,2);
 si = cumsum(earning,1);
 probcdf = cumsum(earprobsort(:,2));
 plot(probcdf, si);
 siminus = [0; si(1:2*nY*nA-1)];
 Gini = 1 - (earprobsort(:,2)'*(si+siminus))/si(2*nY*nA);
 
%finding the earnings quintiles
 quintear = probcdf*ones(1,4) - kron([.2,.4,.6,.8], ones(nA*nY*2,1));
 quintear = quintear < 0;
 quintilesear = [sum((earprobsort(:,1)*ones(1,4)).*quintear.*(earprobsort(:,2)*ones(1,4)))/si(nA*nY*2),1];
 quintilesear = quintilesear - [0, quintilesear(1:4)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%recover the values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%u not needed anywhere else, make sure to use the right parameters
if sigma == 1
   u  = @(c) log(c);
   up = @(c) 1./c;
else
    u  = @(c) c.^(1-sigma)./(1-sigma);
    up = @(c) c.^-sigma;
end

v  = @(l) phi*l.^(1+gamma)./(1+gamma);
vp = @(l) phi*l.^gamma;

valuerec = 0;%handle to recover the value function
if valuerec ==1;

Vs = zeros(nA, nY);
Vu = zeros(nA, nY);    
   
%load Vs Vs
%load Vu Vu    
    
diffv = 1;
xv = 0;
normv = 10^-10;

while diffv > normv

    for k = 1:nA
        for m = 1:nY
                TVs(k,m) = u(cspol(k,m)) - v(lspol(k,m)) + betas*Ts(m,:)*(interp1(A, Vs, asprime(k,m),'linear','extrap'))';
                TVu(k,m) = u(cupol(k,m)) - v(lupol(k,m)) + betau*Tu(m,:)*(interp1(A, Vu, auprime(k,m),'linear','extrap'))';
         end
    end
    diffv = sum(sum(abs(Vs-TVs))) + sum(sum(abs(Vu-TVu)));
    Vs = TVs;
    Vu = TVu;
    xv = xv+1;
end


%save Vs Vs
%save Vu Vu
Vmeans = sum(sum(Vs.*lambdasmat))
Vmeanu = sum(sum(Vu.*lambdaumat))
end

umeans = sum(sum(lambdasmat.*u(cspol)))/(1-betas);
umeanu = sum(sum(lambdaumat.*u(cupol)))/(1-betau);
vmeans = sum(sum(lambdasmat.*v(lspol)))/(1-betas);
vmeanu = sum(sum(lambdaumat.*v(lupol)))/(1-betau);

Vmeans = umeans - vmeans;
Vmeanu = umeanu - vmeanu;
Vmean = Vmeans + Vmeanu;

output = [pis, sigma, gamma, phi, omega, nu,  betas, betau, lambdal, meany, G, B, umeans, umeanu, vmeans, vmeanu, Vmeans, Vmeanu, Vmean, tauks, tauke, ctax, corptax, ...
    cctax, taul, ltax, ats atu, tauc, gdp, G/gdp, B/gdp, ks, ke,  ke+ks, ke/(ke+ks), ...
    (ke+ks)/gdp, assets, cagg, iagg, c1, c2, lagg, l1, l2, w(1), w(2), w(2)/w(1) (lu*w(1) + ls*w(2))/gdp, rks, rs, re, (rs-ds)*(1-tauks), (re-de)*(1-tauke)]';
%save output output
avret = (((rs-ds)*ks + (re-de)*ke)/(ks+ke));
%calibrated parameters as inputs into the Reform3.m file
inputnl = [r, w, lambdal, B, betas, omega, nu, betau, meany, G, phi];
save inputnl inputnl


%productivity gains
tauke = 0;
tauks = 0;
k0 = ke+ks;
xold = x;
%finding the individual ke and ks s.t. the nac holds
options = optimset('display', 'off', 'tolfun', 10^(-20), 'tolX', 10^(-20), 'maxfunevals', 1000000, 'maxiter', 1000000);
ke = fsolve(@nacdiff, .5*k0, options);
ks = k0 - ke;

x(1:2) =  [ks, ke];
x;
gdpalt = ycomp(x);
gdpaltnet = gdpalt - ds*ks - de*ke;

MP = ycompprime2(x);
outputalt = [gdpalt; gdpaltnet; ((MP(1)-ds)*ks+(MP(2)-de)*ke)/(ks+ke); ((MP(1)-ds)*ks+(MP(2)-de)*ke)/(ks+ke)/avret-1];
output = [output; outputalt];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%figures
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figures = 0;
if figures  == 1

figure(1)
plot(A, lambdasmat, 'r');
hold on;
plot(A, lambdaumat, 'b');
title('pdfs')
% hold on;
% plot(A, lambdasa, 'r');
% hold on;
% plot(A, lambdaua, 'b');


% figure(2)
% plot(A, cspol, 'r');
% hold on;
% plot(A, cupol, 'b');
% title('consumption')

 figure(3)
 plot(A, lspol, 'r');
 hold on;
 plot(A, lupol, 'b');
 title('labor')
 
figure(4)
plot(A, lambdasmatcdf, 'r');
hold on;
plot(A, lambdaumatcdf, 'b');
title('cdfs')
% 
 figure(5)
 plot(A, ears, 'r');
 hold on;
 plot(A, earu, 'b');
 title('pre-tax earnings')

%
%  figure(5)
%  plot(A, asprime, 'r');
%  hold on;
%  plot(A, auprime, 'b');
%  hold on;
%  plot(A,A, 'k');

% figure(6)
% plot(A, V(:,1), 'r');
% hold on;
% plot(A, V(:,2), 'b');
% 

end